//
// Projekt Wetterstation
// Datei: buch4_kap10_wetterstation.ino


#include <SPI.h>
#include <Ethernet.h>
#include <TextFinder.h>
#include <LiquidCrystal.h>
#include "DHT.h"


// Ethernet
byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xAD };
byte ip[] = { 10, 0, 1, 101 };
byte gateway[] = { 10, 0, 1, 1 };
byte subnet[] = { 255, 255, 255, 0 };
byte server[] = { 123, 123, 222, 222 }; // Website mit PHP-Skript

// Ethernet-Objekt
EthernetClient client;
// TextFinder-Objekt
TextFinder  finder( client ); 

// DHT-Sensoren
// Innensensor
DHT dht11(16, DHT11);
// Aussensensor
DHT dht22(17, DHT22);

// Buttonvariablen
// Button für Bildumschaltung
const int  buttonPin = 2;
// Bildzähler (1-3)
int buttonPushCounter = 1;


//Initialisierung LCD, Pin-Belegung
LiquidCrystal lcd(8, 9, 4, 5, 6, 7);

// Wartezeiten
long previousMillis1 = 0;
// Current Weather
long interval1 = 60000;
long previousMillis2 = 0;
// Weather Forecast
long interval2 = 60000;
long previousMillis3 = 0;
// Sensor Data
long interval3 = 10000;
long startinterval1=1000;
long startinterval2=1000;
long startinterval3=1000;
long iv1;
long iv2;
long iv3;

// Variablen Current Weather
char now_temp[30];
char now_cond[50];
char now_hum[50];

// Variablen Weather Forecast
char today_day[10];
char today_cond[30];
char today_min[10];
char today_max[10];
char tomorrow_day[10];
char tomorrow_cond[30];
char tomorrow_min[10];
char tomorrow_max[10];

void setup() 
{
  // Inputtaster zur Bildschirmwahl
  pinMode(buttonPin, INPUT);
  // Interrupt
  attachInterrupt(0, Taste, FALLING);
  // Serielle Schnittstelle
  Serial.begin(9600);
  // Start Ethernet-Verbindung
  Ethernet.begin(mac, ip);
  // Start Sensoren DHT
  dht11.begin();
  dht22.begin();
  
  // Ausgabe Startbildschirm
  lcd.begin(20, 4);
  lcd.setCursor(0,0);
  lcd.print("--Weather Station--  ");
  lcd.setCursor(0,1);
  lcd.print("Projekt");
  lcd.setCursor(0,2);
  lcd.print("V1.0");
  delay(1000);
  lcd.clear();
  int buttonPushCounter = 1;
  lcd.setCursor(0,0);
  lcd.print("--Weather Station--  ");
  lcd.setCursor(0,1);
  lcd.print("Read Data...");
  iv1=startinterval1;
  iv2=startinterval2;
  iv3=startinterval3;
}


void loop()
{
  // LCD Current Weather
  if (buttonPushCounter == 1)
  {
    func1();
  }
  
  // LCD Weather Forecast
  if (buttonPushCounter == 2)
  {
   func2();
  }
  
  // LCD Sensor Data
  if (buttonPushCounter == 3)
  {
    
    func3();
  }
}

// Taste abfragen
// Funktion wird durch Interrupt aufgerufen
void Taste()
{
  iv1=startinterval1;
  iv2=startinterval2;
  iv3=startinterval3;
  buttonPushCounter++;
  if (buttonPushCounter == 4)
  {
    buttonPushCounter=1;
  }
  Serial.println(buttonPushCounter, DEC);
}

//
// Funktionen
//

void func1()
{
  unsigned long currentMillis1= millis();

  if(currentMillis1 - previousMillis1 > iv1)
  {
    previousMillis1 = currentMillis1;
    getWeather();
  }
  iv1=interval1;
}

void func2()
{
  unsigned long currentMillis2= millis();
  if(currentMillis2 - previousMillis2 > iv2)
  {
    previousMillis2 = currentMillis2;
    getWeatherForecast();
  }
   iv2=interval2;
}

void func3()
{
  unsigned long currentMillis3= millis();
  if(currentMillis3 - previousMillis3 > iv3)
  {
    previousMillis3 = currentMillis3;
    getWeatherData();
  }
   iv3=interval3;
}

//
// Wetterfunktionen
//

// Weather Forecast
void getWeatherForecast()
{

  if (client.connect(server, 80)>0) {
    Serial.println("Connect Weather XML...");
    client.println("GET /forecast-xml.php HTTP/1.0");
    client.println("HOST:www.meinserver.com\n\n");
    client.println();
    Serial.println("Connected...");
  } 
  else {
    Serial.println(" connection failed");
  } 
  if (client.connected())
  {
    // Wetter XML abfragen
    
   // Wettervorhersage heute
   // Tag
   // Feld in XML: <forecast_today_day>
   finder.getString("<forecast_today_day>", "<forecast_today_day>",today_day,10);
   Serial.print("Day: ");
   Serial.println(today_day);
    
   // Konditionen
   // Feld in XML: <forecast_today_cond>
   finder.getString("<forecast_today_cond>", "<forecast_today_cond>",today_cond,30);
   Serial.print("Today Cond: ");
   Serial.println(today_cond);
   
   // Temp Min
   // Feld in XML: <forecast_today_min>
   finder.getString("<forecast_today_min>", "<forecast_today_min>",today_min,10);
   Serial.print("Today Min: ");
   Serial.println(today_min);
   
   // Temp Max
   // Feld in XML: <forecast_today_max>
   finder.getString("<forecast_today_max>", "<forecast_today_max>",today_max,10);
   Serial.print("Tomorrow Max: ");
   Serial.println(today_max);
   
   
   // Wettervorhersage Morgen
   // Tag
   // Feld in XML: <forecast_tomorrow_day>
   finder.getString("<forecast_tomorrow_day>", "<forecast_tomorrow_day>",tomorrow_day,10);
   Serial.print("Day: ");
   Serial.println(tomorrow_day);
    
   // Konditionen
   // Feld in XML: <forecast_tomorrow_cond>
   finder.getString("<forecast_tomorrow_cond>", "</forecast_tomorrow_cond>",tomorrow_cond,30);
   Serial.print("tomorrow Cond: ");
   Serial.println(tomorrow_cond);
   
   // Temp Min
   // Feld in XML: <forecast_tomorrow_min>
   finder.getString("<forecast_tomorrow_min>", "</forecast_tomorrow_min>",tomorrow_min,10);
   Serial.print("tomorrow Min: ");
   Serial.println(tomorrow_min);
   
   // Temp Max
   // Feld in XML: <forecast_tomorrow_max>
   finder.getString("<forecast_tomorrow_max>", "</forecast_tomorrow_max>",tomorrow_max,10);
   Serial.print("tomorrow Max: ");
   Serial.println(tomorrow_max);
  
   // Ausgabe auf LCD
   lcd.clear();
    // Forecast Today
   lcd.setCursor(0,0);
   lcd.print(today_day);
   lcd.print(":");
   lcd.setCursor(0,1);
   lcd.print(today_cond);
   lcd.print(", ");
   lcd.print(today_min);
   lcd.print("-");
   lcd.print(today_max);
   // Forecast Tomorrow
   lcd.setCursor(0,2);
   lcd.print(tomorrow_day);
   lcd.print(":");
   lcd.setCursor(0,3);
   lcd.print(tomorrow_cond);
   lcd.print(",");
   lcd.print(tomorrow_min);
   lcd.print("-");
   lcd.print(tomorrow_max);
  
   // Ende XML abfragen
 }
  
  else
  {
    // falls keine Verbindung aufgebaut werden konnte
    Serial.println("Disconnected");
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.print("--Weather Forecast--");
    lcd.setCursor(0,1);
    lcd.print("No Data received...");
  }
  // Verbindung Ende
  client.stop();
  client.flush();  

}

// Current Weather
void getWeather()
{
  if (client.connect(server, 80)>0) {
    Serial.println("Connect Weather XML...");
    client.println("GET /weather-xml.php HTTP/1.0");
    client.println("HOST:www.meinserver.com\n\n");
    client.println();
    Serial.println("Connected...");
  } 
  else {
    Serial.println(" connection failed");
  } 
  if (client.connected())
  {
    // Wetter XML abfragen
    
   // Aktuelle Konditionen
   // Feld in XML: <weather_now_cond>
   finder.getString("<weather_now_cond>", "</weather_now_cond>",now_cond,50);
   Serial.print("Cond: ");
   Serial.println(now_cond);
    
    // Aktuelle Temperatur abfragen
    // Feld in XML: <weather_now_temp>
    if ( (finder.getString("<weather_now_temp>", "</weather_now_temp>",now_temp,30)!=0) )
    {
     Serial.print("Temp C:  ");
     Serial.println(now_temp);
   }
   else
   {
     Serial.println("Temp C: ---");
   }
   
   // Aktuelle Luftfeuchtigkeit
   // Feld in XML: <weather_now_hum>
   finder.getString("<weather_now_hum>Humidity: ", "</weather_now_hum>",now_hum,50);
   Serial.print("Hum: ");
   Serial.println(now_hum);
  
   // Ausgabe auf LCD
   lcd.clear();
   lcd.setCursor(0,0);
   lcd.print("--Current Weather--");
   lcd.setCursor(0,1);
   lcd.print("Temp: ");
   lcd.print(now_temp);
   lcd.print(" C ");
   lcd.setCursor(0,2);
   lcd.print("Humidity: ");
   lcd.print(now_hum);
   lcd.setCursor(0,3);
   lcd.print(now_cond);
  
   // Ende XML abfragen
 }
  
  else
  {
    // falls keine Verbindung aufgebaut werden konnte
    Serial.println("Disconnected");
    lcd.clear();
    lcd.setCursor(0,0);
    lcd.print("--Current Weather--");
    lcd.setCursor(0,1);
    lcd.print("No Data received...");
  }
  // Verbindung Ende
  client.stop();
  client.flush();  

}

// Weather Data
void getWeatherData()
{
  // DHT11 Innensensor
  float h11 = dht11.readHumidity();
  float t11 = dht11.readTemperature();
  // DHT22 Aussensensor
  float h22 = dht22.readHumidity();
  float t22 = dht22.readTemperature();
  
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("--Sensor Data--");
  
  if (isnan(t11) || isnan(h11))
  {
   lcd.setCursor(0,1);
   lcd.print("In/Out: ");
   lcd.setCursor(0,2);
   lcd.print("Err: No Sensor data");
  }
  else
  {
    lcd.setCursor(0,1);
    lcd.print("In/Out: ");
    lcd.setCursor(0,2);
    lcd.print("T:");
    lcd.print(t11);
    lcd.print("C H:");
    lcd.print(h11);
    lcd.print("%");
  }
  
  // Ausgabe Aussensensor
  if (isnan(t22) || isnan(h22))
  {
   lcd.setCursor(0,3);
   lcd.print("Err: No Sensor data");
  }
  else
  {
    lcd.setCursor(0,3);
    lcd.print("T:");
    lcd.print(t22);
    lcd.print("C H:");
    lcd.print(h22);
    lcd.print("%");
  }
}
