//
// Kompass mit HMC5883 Sensor
// Datei: buch4_kap5_hmc5883_compass.ino

/*****************************************************************************/	
//	Function:	 Get the Geographic direction of the X-axis.
//				If X-axis points to the North, it is 0 degree.
//				If X-axis points to the East, it is 90 degrees.
//				If X-axis points to the South, it is 180 degrees.
//				If X-axis points to the West, it is 270 degrees.
//  Hardware:   Grove - 3-Axis Digital Compass
//	Arduino IDE: Arduino-1.0
//	Author:	 Frankie.Chu		
//	Date: 	 Jan 10,2013
//	Version: v1.0
//	by www.seeedstudio.com
//
//  This library is free software; you can redistribute it and/or
//  modify it under the terms of the GNU Lesser General Public
//  License as published by the Free Software Foundation; either
//  version 2.1 of the License, or (at your option) any later version.
//
//  This library is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//  Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with this library; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//
/*******************************************************************************/

#include <Wire.h>
#include <HMC5883L.h>

// Kompass Objekt
HMC5883L compass;
// Fehler
int error = 0;


void setup()
{
  // Start serielle Schnittstelle
  Serial.begin(9600);
  Serial.println("Kompass...");
  Wire.begin();

  // Skalierung setzen
  error = compass.SetScale(1.3);
  // falls Fehler
  if(error != 0)
    Serial.println(compass.GetErrorText(error));
  
  // Continuous Mode setzen
  error = compass.SetMeasurementMode(Measurement_Continuous);
  if(error != 0)
    Serial.println(compass.GetErrorText(error));
}


void loop()
{
  // Rohdaten einlesen
  MagnetometerRaw raw = compass.ReadRawAxis();
  // Skalierte Daten
  MagnetometerScaled scaled = compass.ReadScaledAxis();
  // Kalkulation
  float heading = atan2(scaled.YAxis, scaled.XAxis);
  
  // Ermittlung Magnetic Declination unter http://www.magnetic-declination.com/
  // Mein Wert: +2° 29' -> 2.48333° -> Rad 0.043342
  float declinationAngle = 0.043342;
  heading += declinationAngle;
  
  // Korrektur wenn invertiert
  if(heading < 0)
    heading += 2*PI;
  // Limitierung Winkel
  if(heading > 2*PI)
    heading -= 2*PI;
   
  // Umrechnung Radiant zu Grad
  float headingDegrees = heading * 180/M_PI; 

  // Ausgabe Winkel
  Serial.print(headingDegrees);
  Serial.println(" Grad");

  delay(100);
}
