//
// Projekt Netzteil mit Encoder
// Gesamte Funktion (Rotary Encoder für U/I Eingabe, EEPROM, Ausgabe PWM)
// Datei: buch4_kap8_dig_netzteil_encoder.ino


#include <EEPROM.h>

// Pindeklaration
int PinEncSU=8;

// Encoder
int encoderPinA = 2;       // Encoder Signal A
int encoderPinB = 4;       // Encoder Signal B
int interruptChannel = 0;  // Interrupt 0 = Pin 2
int stepSize = 1;          // Steps für Veränderung Wert

// Sollwert U 
volatile uint8_t valSollU;
float SollU;

// PWM U
int PinPWMU = 9;

// EEPROM
int eeaddressU = 0;
byte eevalueU;

static void interruptHandler()
{
  // Verzögerung als Debounce
  delay(30);                       
  if ( digitalRead( encoderPinA ) != digitalRead( encoderPinB ) ) 
  {
    if ( valSollU <= ( 255 - stepSize ) ) 
    {
      valSollU += stepSize;
    }
  } else {
    if ( valSollU >= stepSize ) 
    {
      valSollU -= stepSize;
    }
  } 
}


void setup()
{
  // Serielle Schnittstelle Start
  Serial.begin(9600);
  // Encoder, Eingänge Pull-up aktivieren
  pinMode( encoderPinA , INPUT_PULLUP );
  pinMode( encoderPinB , INPUT_PULLUP );
  // Ausgang PWM U
  pinMode(PinPWMU, OUTPUT );
  // Eingang Soll U speichern
  pinMode(PinEncSU, INPUT);
  delay(500);
  // EEPROM Wert lesen
  eevalueU = EEPROM.read(eeaddressU);
  valSollU=eevalueU;
  analogWrite( PinPWMU , valSollU );
  // Interrupt
  attachInterrupt( interruptChannel , interruptHandler , CHANGE );
}

void loop()
{
  // Wert Speichern
  if (digitalRead(PinEncSU) == HIGH)
  {
    // Wert in EEPROM schreiben
    EEPROM.write(eeaddressU, valSollU);
    Serial.println("Sollwert gespeichert");
  }
  // Ausgabe Sollwert U
  Serial.print("U: ");
  SollU=valSollU;
  SollU=SollU/10;
  Serial.println(SollU);
  
  // Ausgabe Sollwert U
  analogWrite(PinPWMU, valSollU);
  delay(200);
}


