//
// MCP4725 DAC - Analog-Spannung ausgeben
// Datei: buch4_kap8_mcp4725_sollwert.ino

#include <Wire.h>
#include <Adafruit_MCP4725.h>
#include <EEPROM.h>


// DAC
Adafruit_MCP4725 dac;

// Encoder
// Speicher
int PinEncSU=8;
// Signale
int encoderPinA = 2;       // Encoder Signal A
int encoderPinB = 4;       // Encoder Signal B
int interruptChannel = 0;  // Interrupt 0 = Pin 2
int stepSize = 1;          // Steps für Veränderung Wert

// Sollwert U 
volatile uint8_t valSollU;
float SollU;

// PWM U
int PinPWMU = 9;

// EEPROM
int eeaddressU = 0;
byte eevalueU;

static void interruptHandler()
{
  // Verzögerung als Debounce
  delay(30);                       
  if ( digitalRead( encoderPinA ) != digitalRead( encoderPinB ) ) 
  {
    if ( valSollU <= ( 255 - stepSize ) ) 
    {
      valSollU += stepSize;
    }
  } else {
    if ( valSollU >= stepSize ) 
    {
      valSollU -= stepSize;
    }
  } 
}

void setup(void)
{
  Serial.begin(9600);
  // Encoder, Eingänge Pull-up aktivieren
  pinMode( encoderPinA , INPUT_PULLUP );
  pinMode( encoderPinB , INPUT_PULLUP );
  // Eingang Soll U speichern
  pinMode(PinEncSU, INPUT);
  // DAC starten
  dac.begin(0x62);
  Serial.println("Start MCP4725");  
  Serial.println("Sollwertgeber...");
  // EEPROM Wert lesen
  eevalueU = EEPROM.read(eeaddressU);
  valSollU=eevalueU;
  dac.setVoltage(valSollU*10, false);  
  // Interrupt
  attachInterrupt( interruptChannel , interruptHandler , CHANGE );
}

void loop(void)
{
  // Wert Speichern
  if (digitalRead(PinEncSU) == HIGH)
  {
    // Wert in EEPROM schreiben
    EEPROM.write(eeaddressU, valSollU);
    Serial.println("Sollwert gespeichert");
  }
  // Ausgabe Sollwert
  Serial.print("Vout: ");
  Serial.println(valSollU*10);
  dac.setVoltage(valSollU*10, false);  
 delay(50); 
}
